/*
 *	NET3:	Implementation of the squashed header stack.
 *	
 *		Alan Cox, <A.Cox@swansea.ac.uk / gw4pts@gw4pts.ampr.org>
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version
 *	2 of the License, or (at your option) any later version.
 *
 *	
 *	This is based upon Van Jaconson's idea of a 'squashed stack'. Most
 *	of the time we don't actually need to do any work on packet headers,
 *	nor search arp headers, but can just use 'one we prepared earlier'.
 */

static unsigned short dummy;	/* Used to save time when we invaliate a cache slot with no inform
				   address set */

unsigned short hdr_alloc(int bytes)
{
	unsigned short ct;
	unsigned long flags;
	struct hdr_cache *mhp;
	
	bytes+=sizeof(*mhp);
	mhp=(struct hdr_cache *)kmalloc(bytes,  GFP_ATOMIC);
	if(mhp==NULL)
		return -1;
	
	mhp->family= -1;	/* Family that will not match */
	mhp->locked = 1;	/* Won't go away (yet) */
	mhp->flags = 0;
	mhp->dev = NULL;
	mhp->inval_ptr = &dummy;	/* Global dummy value */
	init_timer(&mhp->expire_time);
	
	save_flags(flags);
	cli();
	for(ct=0;ct<NUM_HDR_SLOTS;ct++)
	{
		if(hdr_pointer[ct]==NULL)
		{
			hdr_pointer[ct]=mhp;
			mhp->slot = ct;
			restore_flags(flags);
			return ct;
		}
		ct++;
	}
	kfree_s(mhp, bytes);
	return -1;
}
		
/*
 *	Free a cache entry
 */
 
void hdr_free(struct hdr_cache *hd)
{
	if(hd->locked)
	{
		/*
		 *	Busy so defer timer
		 */
		save_flags(flags);
		cli();
		if(hd->flags&HDC_HASTIMER)
			del_timer(&hd->timer);
		hd->flags|=HDC_HASTIMER|HDC_DEAD;
		hd->timer.expires= 10;	/* Very soon */
		hd->timer.function= &hdr_expire;
		hd->timer.data= (unsigned long)hd;
		if(hd->slot!= -1)
		{
			hd_pointer[hd->slot]=NULL;
			hd->slot = -1;
		}
		add_timer(&hd->timer);
		restore_flags(flags);
		return;
	}
	*hd->inval_ptr= -1;	/* Invalidate back reference */
	restore_flags(flags);
	kfree(hd);
}

