/*
 *	NET3:	Definition of the squashed header cache
 *	
 *		Alan Cox, <A.Cox@swansea.ac.uk / gw4pts@gw4pts.ampr.org>
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version
 *	2 of the License, or (at your option) any later version.
 *
 *	
 *	This is based upon Van Jaconson's idea of a 'squashed stack'. Most
 *	of the time we don't actually need to do any work on packet headers,
 *	nor search arp headers, but can just use 'one we prepared earlier'.
 */

#ifndef __HDR_CACHE_H
#define __HDR_CACHE_H
 
struct hdr_cache
{
 	unsigned short	family;		/* Socket family */
 	unsigned short	slot;		/* Our slot */
 	char	 key[MAX_HD_KEY];	/* Key size for searches (avoid if possible) */
 	unsigned char   keysize;	/* Size of the key */
 	volatile unsigned char locked;	/* Not permitted to go away (in use) */
 	unsigned short	flags;		/* Flags */
#define HDC_HASTIMER	1		/* Has an expiry timer */
#define HDC_DEAD	2		/* Dead but not yet deleted */ 	
 	struct   device *dev;		/* Owner device (used to expire entries on dev change/loss */
 	unsigned short   *inval_ptr;	/* When we invalidate a record set this address to -1 if not NULL
 					   allows us to avoid hard searches on invalidates */
 	int	 writes;		/* Number of layers that have to make header changes. As soon as it hits zero
 					   on a pass down just hurl the buffer at the device */
 	struct timer	expire_time;	/* Expiry timer for time expire */
 	unsigned char	data[0];	/* The header */
};
 
#define NUM_HDR_SLOTS		64
 
extern struct hdr_cache hdr_pointer[MAX_HDR_SLOTS];
unsigned short hdr_alloc(int bytes);
void hdr_free(struct hdr_cache *hd);

static inline struct hdr_cache *hdr_get(unsigned short slot) { return hdr_pointer[slot]; }

 
#endif